#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>
#include "articulos.h"
#include "idxarticulos.h"


#define  MAXREGIDX     100000  // Mximo nmero de elemento en la tabla indice

/* Tabla de indices  */
struct ElementoIdx {
           char clave[10];
           long posicion;
           };

struct ElementoIdx TablaIdx [MAXREGIDX];   // Tabla de indices
static int numregidx;                            // Nmero de claves almacendas



/* -----------------------------------------------------------------------*/
/* -----------------------------------------------------------------------*/
/*  FUNCIONES DE MANEJO  DE LA TABLA DE INDICES                           */
/* -----------------------------------------------------------------------*/
/* -----------------------------------------------------------------------*/

// FUNCIONES PRIVADAS:


static int Idx_BuscarEnTabla ( char *clave );

// Funcin que compara dos Elementos de la tabla de indices
// Parmetro necesario de la fncin de liberia: qsort

/* -----------------------------------------------------------------------
 * FUNCION:
 * DESCRIPCION:
 * PARAMETROS:
 *
 * DEVUELVE
 *
 * ----------------------------------------------------------------------*/

static int micomp ( const void  *n1, const void  *n2)
{
  return strcmp( (( struct ElementoIdx *) n1)->clave,
                 (( struct ElementoIdx *) n2)->clave );
}


//  FUNCIONES PUBLICAS:

/* -----------------------------------------------------------------------
 * FUNCION:  void Idx_InitTabla ( FILE *f )
 * DESCRIPCION: Inicia las tabla de indices leyendo todo el fichero
 *  y ordenando la tabla por el mtodo rpido funcin de librera:qsort
 * PARAMETROS :
 *    f (IN) : fichero del que se genera el ndice
 * DEVUELVE
 *     NADA
 * ---------------------------------------------------------------------- */
// PROBLEMA: Esta funcin esta muy acoplada a articulos
void Idx_InitTabla ( FILE *f )
{
   long pos,posold;
   TipoArticulo reg;
   char dib[5] = "-\\|/"; // Tonteria animada
   int i;

   clrscr();

   cprintf(" Cargando indice : ");

   /* Guardo la antigua posicion y me situo al principio */
   posold = ftell(f);
	rewind(f);

   i=0;
	fread(&reg,sizeof(reg),1,f);
	while(!feof(f))
	{
      // No guardo los registros borrados
      if ( reg.grupo != 0 )
         {
		   pos = ftell(f)-(sizeof(reg));
         TablaIdx[i].posicion = pos;
         strcpy(TablaIdx[i].clave,reg.codigo);
         i++;
         // Muestro la Tonteria animada
         gotoxy(30,1); cprintf("%6d:%c",i,dib[i % 4]);
         }
      fread(& reg,sizeof(reg),1,f);
 	}

   // Ordeno la tabla
   qsort(TablaIdx,i,sizeof(struct ElementoIdx),micomp);
   numregidx = i;  // Guardo el nmero de claves almacenadas

   /* Vuelvo a situarme en la antigua posicin */
   fseek(f,posold,SEEK_SET);

   cprintf(" Indice cargado.");
}
/* -----------------------------------------------------------------------
 * FUNCION: int Idx_Anotar ( char *clave, long posicion )
 * DESCRIPCION: Anota ordenadamente una clave en la tabla, devuelve 0
 *   si no hay sitio o si la clave est repetida.
 * PARAMETROS:
 *   - clave (IN): clave a almacenar
 * DEVUELVE
 *   1 si xito
 *   0 si la tabla esta llena
 *  -1 si la clave est repetida
 * ----------------------------------------------------------------------*/

int Idx_Anotar ( char *clave, long posicion )
{
   int I;
   struct ElementoIdx nuevo;

   // Si estoy a tope de elementos
	if ( numregidx == MAXREGIDX )
       {
       return 0;
       }

   // Compruebo si el elemento no est repetido
   if (Idx_BuscarEnTabla( clave ) != -1 )
       {
       return -1;
       }

   strcpy(nuevo.clave,clave);
   nuevo.posicion = posicion;

  if(numregidx == 0)
  	{
   	TablaIdx[0]=nuevo;
  	}
   else
   {
   I = numregidx - 1;
	while((strcmp(nuevo.clave,TablaIdx[I].clave)<0)&& (I > 0 ))
   	{
      TablaIdx[I+1] = TablaIdx [I]; // Desplazo los elementos mayores
      I--;
   	}

   // Situo el nuevo elemento
   if ( strcmp(TablaIdx[I].clave,nuevo.clave ) > 0)
      {
      TablaIdx[I+1] = TablaIdx[I];
      TablaIdx[I] = nuevo;
      }
   else
      {
      TablaIdx[I+1] = nuevo;
      }
   }

   numregidx++; // Incremento el nmero de elemento de la tabla indice
   return 1;

}

/* -----------------------------------------------------------------------
 * FUNCION: int Idx_Borrar ( char *clave )
 * DESCRIPCION: Borra una clave de la tabla de indices
 * PARAMETROS:
 *    - clave (IN): Clave de registro a buscar
 * DEVUELVE
 *    1 si xito o 0 si fallo
 * ----------------------------------------------------------------------*/

int Idx_Borrar ( char *clave )
{
  int posi;
  int i;

  // Obtengo la posicin en la tabla
  posi = Idx_BuscarEnTabla ( clave );
  if ( posi != -1 )
     {
     /* Desplazo los elementos, machacando el antiguo */
     for (i= posi; i < numregidx -1 ; i++ )
        {
        TablaIdx[i] = TablaIdx[i+1];
        }
     numregidx--; // Decremento el nmero de elementos en el indice
     return 1;
     }
  else
     {
     return 0;
     }

}
/* -----------------------------------------------------------------------
 * FUNCION: Idx_BuscarEnTabla ( char *clave )
 * DESCRIPCION:  Realiza una busqueda dicotmica en la tabla
 * PARAMETROS
 *  - CLAVE (IN): Clave a buscar en el indice
 * DEVUELVE:
 *   La posicin en la tabla o -1 si no lo encuentra
 * ----------------------------------------------------------------------*/

static int Idx_BuscarEnTabla ( char *clave )
{
  int encontrado=0;
  int izda = 0;
  int dcha = numregidx - 1;
  int resu = -1; // No est
  int resucmp;
  int medio;

  while ((encontrado==0)&&(izda <= dcha))
    	{
      medio=( izda + dcha)/2;
      resucmp = strcmp(TablaIdx[medio].clave,clave);
      if ( resucmp == 0)
   	  {
        encontrado=1;
        }
     else
        {
        if ( resucmp > 0)
          	{
            dcha=medio-1;
            }
        else
         	{
            izda=medio+1;
            }
        }
     }

  if ( encontrado )
      {
      resu = medio; /* Guarda la posicin */
      }

 return resu;
}

/* -----------------------------------------------------------------------
 * FUNCION: long Idx_Buscar ( char *clave)
 * DESCRIPCION: Busca en el ndice la posicin fsica en el fichero
    del registro con la clave indicada
 * PARAMETROS:
 *   clave (IN): Clave del registro a buscar
 * DEVUELVE
 *   La posicin fsica del registro
 *   -1 si no se encuentra
 * ----------------------------------------------------------------------*/

long Idx_Buscar ( char *clave)
{
   long pos ; // Posicin dentro del fichero
   int  posi; // Posicin en la tabla de indices

   pos = -1; // No est
   posi = Idx_BuscarEnTabla( clave );
   if ( posi != -1 )
      {
      pos = TablaIdx[posi].posicion;
      }

   return pos;
}


